# IO.Swagger - the C# library for the Weather Source History API

Daily and hourly historical weather data for latitude/longitude points, Zip/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source History API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 

This C# SDK is automatically generated by the [Swagger Codegen](https://github.com/swagger-api/swagger-codegen) project:

- API version: 2.0.0
- SDK version: 1.0.0
- Build package: io.swagger.codegen.v3.generators.dotnet.CSharpClientCodegen
    For more information, please visit [https://developer.weathersourceapis.com/](https://developer.weathersourceapis.com/)

<a name="frameworks-supported"></a>
## Frameworks supported
- .NET 4.0 or later
- Windows Phone 7.1 (Mango)

<a name="dependencies"></a>
## Dependencies
- [RestSharp](https://www.nuget.org/packages/RestSharp) - 105.1.0 or later
- [Json.NET](https://www.nuget.org/packages/Newtonsoft.Json/) - 7.0.0 or later
- [JsonSubTypes](https://www.nuget.org/packages/JsonSubTypes/) - 1.2.0 or later

The DLLs included in the package may not be the latest version. We recommend using [NuGet](https://docs.nuget.org/consume/installing-nuget) to obtain the latest version of the packages:
```
Install-Package RestSharp
Install-Package Newtonsoft.Json
Install-Package JsonSubTypes
```

NOTE: RestSharp versions greater than 105.1.0 have a bug which causes file uploads to fail. See [RestSharp#742](https://github.com/restsharp/RestSharp/issues/742)

<a name="installation"></a>
## Installation
Run the following command to generate the DLL
- [Mac/Linux] `/bin/sh build.sh`
- [Windows] `build.bat`

Then include the DLL (under the `bin` folder) in the C# project, and use the namespaces:
```csharp
using IO.Swagger.Api;
using IO.Swagger.Client;
using IO.Swagger.Model;
```
<a name="packaging"></a>
## Packaging

A `.nuspec` is included with the project. You can follow the Nuget quickstart to [create](https://docs.microsoft.com/en-us/nuget/quickstart/create-and-publish-a-package#create-the-package) and [publish](https://docs.microsoft.com/en-us/nuget/quickstart/create-and-publish-a-package#publish-the-package) packages.

This `.nuspec` uses placeholders from the `.csproj`, so build the `.csproj` directly:

```
nuget pack -Build -OutputDirectory out IO.Swagger.csproj
```

Then, publish to a [local feed](https://docs.microsoft.com/en-us/nuget/hosting-packages/local-feeds) or [other host](https://docs.microsoft.com/en-us/nuget/hosting-packages/overview) and consume the new package via Nuget as usual.

<a name="getting-started"></a>
## Getting Started

```csharp
using System;
using System.Diagnostics;
using IO.Swagger.Api;
using IO.Swagger.Client;
using IO.Swagger.Model;

namespace Example
{
    public class Example
    {
        public void main()
        {
            // Configure API key authorization: ApiKeyAuth
            Configuration.Default.ApiKey.Add("X-API-KEY", "YOUR_API_KEY");
            // Uncomment below to setup prefix (e.g. Bearer) for API key, if needed
            // Configuration.Default.ApiKeyPrefix.Add("X-API-KEY", "Bearer");

            var apiInstance = new DmasApi();
            var dmaId = 56;  // int? | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
            var date = 2013-10-20;  // DateTime? | An date formatted as a RFC3339 date value.
            var fields = new List<string>(); // List<string> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
            var unitScale = unitScale_example;  // string | The unit scale for returned values.<br> (optional)  (default to IMPERIAL)

            try
            {
                // A day of weather history for a DMA.
                HistoryDmaDayObj result = apiInstance.DmasDmaIdDaysDateGet(dmaId, date, fields, unitScale);
                Debug.WriteLine(result);
            }
            catch (Exception e)
            {
                Debug.Print("Exception when calling DmasApi.DmasDmaIdDaysDateGet: " + e.Message );
            }
        }
    }
}
```

<a name="documentation-for-api-endpoints"></a>
## Documentation for API Endpoints

All URIs are relative to *https://history.weathersourceapis.com/v2*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*DmasApi* | [**DmasDmaIdDaysDateGet**](docs/DmasApi.md#dmasdmaiddaysdateget) | **GET** /dmas/{dmaId}/days/{date} | A day of weather history for a DMA.
*DmasApi* | [**DmasDmaIdDaysDateStartdateEndGet**](docs/DmasApi.md#dmasdmaiddaysdatestartdateendget) | **GET** /dmas/{dmaId}/days/{dateStart},{dateEnd} | A contiguous range of daily weather history for a DMA.
*DmasApi* | [**DmasDmaIdHoursTimestampGet**](docs/DmasApi.md#dmasdmaidhourstimestampget) | **GET** /dmas/{dmaId}/hours/{timestamp} | An hour of weather history for a DMA
*DmasApi* | [**DmasDmaIdHoursTimestampStarttimestampEndGet**](docs/DmasApi.md#dmasdmaidhourstimestampstarttimestampendget) | **GET** /dmas/{dmaId}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly weather history for a DMA.
*OnpointsApi* | [**OnpointsOnpointIdDaysDateGet**](docs/OnpointsApi.md#onpointsonpointiddaysdateget) | **GET** /onpoints/{onpointId}/days/{date} | A day of weather history for an OnPoint™ point.
*OnpointsApi* | [**OnpointsOnpointIdDaysDateStartdateEndGet**](docs/OnpointsApi.md#onpointsonpointiddaysdatestartdateendget) | **GET** /onpoints/{onpointId}/days/{dateStart},{dateEnd} | A contiguous range of daily weather history for an OnPoint™ point.
*OnpointsApi* | [**OnpointsOnpointIdHoursTimestampGet**](docs/OnpointsApi.md#onpointsonpointidhourstimestampget) | **GET** /onpoints/{onpointId}/hours/{timestamp} | An hour of weather history for an OnPoint™ point.
*OnpointsApi* | [**OnpointsOnpointIdHoursTimestampStarttimestampEndGet**](docs/OnpointsApi.md#onpointsonpointidhourstimestampstarttimestampendget) | **GET** /onpoints/{onpointId}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly weather history for an OnPoint™ point.
*PointsApi* | [**PointsLatitudelongitudeDaysDateGet**](docs/PointsApi.md#pointslatitudelongitudedaysdateget) | **GET** /points/{latitude},{longitude}/days/{date} | A day of weather history for a Latitude/Longitude point.
*PointsApi* | [**PointsLatitudelongitudeDaysDateStartdateEndGet**](docs/PointsApi.md#pointslatitudelongitudedaysdatestartdateendget) | **GET** /points/{latitude},{longitude}/days/{dateStart},{dateEnd} | A contiguous range of daily weather history for a Latitude/Longitude point.
*PointsApi* | [**PointsLatitudelongitudeHoursTimestampGet**](docs/PointsApi.md#pointslatitudelongitudehourstimestampget) | **GET** /points/{latitude},{longitude}/hours/{timestamp} | An hour of weather history for a Latitude/Longitude point.
*PointsApi* | [**PointsLatitudelongitudeHoursTimestampStarttimestampEndGet**](docs/PointsApi.md#pointslatitudelongitudehourstimestampstarttimestampendget) | **GET** /points/{latitude},{longitude}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly weather history for a Latitude/Longitude point.
*PostcodesApi* | [**PostcodesPostcodecountryCodeDaysDateGet**](docs/PostcodesApi.md#postcodespostcodecountrycodedaysdateget) | **GET** /postcodes/{postcode},{countryCode}/days/{date} | A day of weather history for a postcode.
*PostcodesApi* | [**PostcodesPostcodecountryCodeDaysDateStartdateEndGet**](docs/PostcodesApi.md#postcodespostcodecountrycodedaysdatestartdateendget) | **GET** /postcodes/{postcode},{countryCode}/days/{dateStart},{dateEnd} | A contiguous range of daily weather history for a postcode.
*PostcodesApi* | [**PostcodesPostcodecountryCodeHoursTimestampGet**](docs/PostcodesApi.md#postcodespostcodecountrycodehourstimestampget) | **GET** /postcodes/{postcode},{countryCode}/hours/{timestamp} | An hour of weather history for a postcode.
*PostcodesApi* | [**PostcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet**](docs/PostcodesApi.md#postcodespostcodecountrycodehourstimestampstarttimestampendget) | **GET** /postcodes/{postcode},{countryCode}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly weather history for a postcode.

<a name="documentation-for-models"></a>
## Documentation for Models

 - [Model.BoundingPoints](docs/BoundingPoints.md)
 - [Model.BoundingPointsInner](docs/BoundingPointsInner.md)
 - [Model.CldCvr](docs/CldCvr.md)
 - [Model.CldCvrAvg](docs/CldCvrAvg.md)
 - [Model.CldCvrMax](docs/CldCvrMax.md)
 - [Model.CldCvrMin](docs/CldCvrMin.md)
 - [Model.CountryCode](docs/CountryCode.md)
 - [Model.CountryName](docs/CountryName.md)
 - [Model.Date](docs/Date.md)
 - [Model.DateEnd](docs/DateEnd.md)
 - [Model.DateRangeObj](docs/DateRangeObj.md)
 - [Model.DateStart](docs/DateStart.md)
 - [Model.DewPt](docs/DewPt.md)
 - [Model.DewPtAvg](docs/DewPtAvg.md)
 - [Model.DewPtMax](docs/DewPtMax.md)
 - [Model.DewPtMin](docs/DewPtMin.md)
 - [Model.Distance](docs/Distance.md)
 - [Model.DistancePop](docs/DistancePop.md)
 - [Model.DmaId](docs/DmaId.md)
 - [Model.DmaName](docs/DmaName.md)
 - [Model.DmaObj](docs/DmaObj.md)
 - [Model.ErrorObj](docs/ErrorObj.md)
 - [Model.FeelsLike](docs/FeelsLike.md)
 - [Model.FeelsLikeAvg](docs/FeelsLikeAvg.md)
 - [Model.FeelsLikeMax](docs/FeelsLikeMax.md)
 - [Model.FeelsLikeMin](docs/FeelsLikeMin.md)
 - [Model.FieldListDayObj](docs/FieldListDayObj.md)
 - [Model.FieldListHourObj](docs/FieldListHourObj.md)
 - [Model.Fields](docs/Fields.md)
 - [Model.FreezingRainFlag](docs/FreezingRainFlag.md)
 - [Model.FreezingRainFlagCnt](docs/FreezingRainFlagCnt.md)
 - [Model.Grid](docs/Grid.md)
 - [Model.HeatIndex](docs/HeatIndex.md)
 - [Model.HeatIndexAvg](docs/HeatIndexAvg.md)
 - [Model.HeatIndexMax](docs/HeatIndexMax.md)
 - [Model.HeatIndexMin](docs/HeatIndexMin.md)
 - [Model.HistoryDayArr](docs/HistoryDayArr.md)
 - [Model.HistoryDayObj](docs/HistoryDayObj.md)
 - [Model.HistoryDmaDayObj](docs/HistoryDmaDayObj.md)
 - [Model.HistoryDmaHourObj](docs/HistoryDmaHourObj.md)
 - [Model.HistoryHourArr](docs/HistoryHourArr.md)
 - [Model.HistoryHourObj](docs/HistoryHourObj.md)
 - [Model.HistoryOnpointDayObj](docs/HistoryOnpointDayObj.md)
 - [Model.HistoryOnpointHourObj](docs/HistoryOnpointHourObj.md)
 - [Model.HistoryPointDayObj](docs/HistoryPointDayObj.md)
 - [Model.HistoryPointHourObj](docs/HistoryPointHourObj.md)
 - [Model.HistoryPostcodeDayObj](docs/HistoryPostcodeDayObj.md)
 - [Model.HistoryPostcodeHourObj](docs/HistoryPostcodeHourObj.md)
 - [Model.IcePelletsFlag](docs/IcePelletsFlag.md)
 - [Model.IcePelletsFlagCnt](docs/IcePelletsFlagCnt.md)
 - [Model.Latitude](docs/Latitude.md)
 - [Model.Longitude](docs/Longitude.md)
 - [Model.MslPres](docs/MslPres.md)
 - [Model.MslPresAvg](docs/MslPresAvg.md)
 - [Model.MslPresMax](docs/MslPresMax.md)
 - [Model.MslPresMin](docs/MslPresMin.md)
 - [Model.OnpointId](docs/OnpointId.md)
 - [Model.OnpointObj](docs/OnpointObj.md)
 - [Model.PointObj](docs/PointObj.md)
 - [Model.Population](docs/Population.md)
 - [Model.Postcode](docs/Postcode.md)
 - [Model.PostcodeObj](docs/PostcodeObj.md)
 - [Model.Precip](docs/Precip.md)
 - [Model.PrecipHourCnt](docs/PrecipHourCnt.md)
 - [Model.PresTend](docs/PresTend.md)
 - [Model.PresTendAvg](docs/PresTendAvg.md)
 - [Model.PresTendMax](docs/PresTendMax.md)
 - [Model.PresTendMin](docs/PresTendMin.md)
 - [Model.ProvisionalFlag](docs/ProvisionalFlag.md)
 - [Model.RadSolar](docs/RadSolar.md)
 - [Model.RadSolarAvg](docs/RadSolarAvg.md)
 - [Model.RadSolarMax](docs/RadSolarMax.md)
 - [Model.RadSolarMin](docs/RadSolarMin.md)
 - [Model.RadSolarTot](docs/RadSolarTot.md)
 - [Model.RainFlag](docs/RainFlag.md)
 - [Model.RainFlagCnt](docs/RainFlagCnt.md)
 - [Model.RelHum](docs/RelHum.md)
 - [Model.RelHumAvg](docs/RelHumAvg.md)
 - [Model.RelHumMax](docs/RelHumMax.md)
 - [Model.RelHumMin](docs/RelHumMin.md)
 - [Model.SamplePoints](docs/SamplePoints.md)
 - [Model.SamplePointsInner](docs/SamplePointsInner.md)
 - [Model.SfcPres](docs/SfcPres.md)
 - [Model.SfcPresAvg](docs/SfcPresAvg.md)
 - [Model.SfcPresMax](docs/SfcPresMax.md)
 - [Model.SfcPresMin](docs/SfcPresMin.md)
 - [Model.SnowDepth](docs/SnowDepth.md)
 - [Model.SnowFlag](docs/SnowFlag.md)
 - [Model.SnowFlagCnt](docs/SnowFlagCnt.md)
 - [Model.Snowfall](docs/Snowfall.md)
 - [Model.SnowfallHourCnt](docs/SnowfallHourCnt.md)
 - [Model.SpcHum](docs/SpcHum.md)
 - [Model.SpcHumAvg](docs/SpcHumAvg.md)
 - [Model.SpcHumMax](docs/SpcHumMax.md)
 - [Model.SpcHumMin](docs/SpcHumMin.md)
 - [Model.SubdivCode](docs/SubdivCode.md)
 - [Model.SubdivName](docs/SubdivName.md)
 - [Model.Temp](docs/Temp.md)
 - [Model.TempAvg](docs/TempAvg.md)
 - [Model.TempMax](docs/TempMax.md)
 - [Model.TempMin](docs/TempMin.md)
 - [Model.Timestamp](docs/Timestamp.md)
 - [Model.TimestampEnd](docs/TimestampEnd.md)
 - [Model.TimestampRangeObj](docs/TimestampRangeObj.md)
 - [Model.TimestampStart](docs/TimestampStart.md)
 - [Model.Timezone](docs/Timezone.md)
 - [Model.Vis](docs/Vis.md)
 - [Model.VisAvg](docs/VisAvg.md)
 - [Model.VisMax](docs/VisMax.md)
 - [Model.VisMin](docs/VisMin.md)
 - [Model.WetBulb](docs/WetBulb.md)
 - [Model.WetBulbAvg](docs/WetBulbAvg.md)
 - [Model.WetBulbMax](docs/WetBulbMax.md)
 - [Model.WetBulbMin](docs/WetBulbMin.md)
 - [Model.WindChill](docs/WindChill.md)
 - [Model.WindChillAvg](docs/WindChillAvg.md)
 - [Model.WindChillMax](docs/WindChillMax.md)
 - [Model.WindChillMin](docs/WindChillMin.md)
 - [Model.WindDir](docs/WindDir.md)
 - [Model.WindDir100m](docs/WindDir100m.md)
 - [Model.WindDir100mAvg](docs/WindDir100mAvg.md)
 - [Model.WindDir80m](docs/WindDir80m.md)
 - [Model.WindDir80mAvg](docs/WindDir80mAvg.md)
 - [Model.WindDirAvg](docs/WindDirAvg.md)
 - [Model.WindSpd](docs/WindSpd.md)
 - [Model.WindSpd100m](docs/WindSpd100m.md)
 - [Model.WindSpd100mAvg](docs/WindSpd100mAvg.md)
 - [Model.WindSpd100mMax](docs/WindSpd100mMax.md)
 - [Model.WindSpd100mMin](docs/WindSpd100mMin.md)
 - [Model.WindSpd80m](docs/WindSpd80m.md)
 - [Model.WindSpd80mAvg](docs/WindSpd80mAvg.md)
 - [Model.WindSpd80mMax](docs/WindSpd80mMax.md)
 - [Model.WindSpd80mMin](docs/WindSpd80mMin.md)
 - [Model.WindSpdAvg](docs/WindSpdAvg.md)
 - [Model.WindSpdMax](docs/WindSpdMax.md)
 - [Model.WindSpdMin](docs/WindSpdMin.md)

<a name="documentation-for-authorization"></a>
## Documentation for Authorization

<a name="ApiKeyAuth"></a>
### ApiKeyAuth

- **Type**: API key
- **API key parameter name**: X-API-KEY
- **Location**: HTTP header

