/* 
 * Weather Source History API
 *
 * Daily and hourly historical weather data for latitude/longitude points, Zip/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source History API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using RestSharp;
using IO.Swagger.Client;
using IO.Swagger.Model;

namespace IO.Swagger.Api
{
    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
        public interface IPointsApi : IApiAccessor
    {
        #region Synchronous Operations
        /// <summary>
        /// A day of weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a single day of historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="date">An date formatted as a RFC3339 date value.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>HistoryPointDayObj</returns>
        HistoryPointDayObj PointsLatitudelongitudeDaysDateGet (float? latitude, float? longitude, DateTime? date, List<string> fields, string unitScale = null);

        /// <summary>
        /// A day of weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a single day of historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="date">An date formatted as a RFC3339 date value.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>ApiResponse of HistoryPointDayObj</returns>
        ApiResponse<HistoryPointDayObj> PointsLatitudelongitudeDaysDateGetWithHttpInfo (float? latitude, float? longitude, DateTime? date, List<string> fields, string unitScale = null);
        /// <summary>
        /// A contiguous range of daily weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a contiguous range of daily historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="dateStart">An date formatted as a RFC3339 date value that must occur temporally before dateEnd.</param>
        /// <param name="dateEnd">An date formatted as a RFC3339 date value that must occur temporally after dateStart.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>HistoryPointDayObj</returns>
        HistoryPointDayObj PointsLatitudelongitudeDaysDateStartdateEndGet (float? latitude, float? longitude, DateTime? dateStart, DateTime? dateEnd, List<string> fields, string unitScale = null);

        /// <summary>
        /// A contiguous range of daily weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a contiguous range of daily historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="dateStart">An date formatted as a RFC3339 date value that must occur temporally before dateEnd.</param>
        /// <param name="dateEnd">An date formatted as a RFC3339 date value that must occur temporally after dateStart.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>ApiResponse of HistoryPointDayObj</returns>
        ApiResponse<HistoryPointDayObj> PointsLatitudelongitudeDaysDateStartdateEndGetWithHttpInfo (float? latitude, float? longitude, DateTime? dateStart, DateTime? dateEnd, List<string> fields, string unitScale = null);
        /// <summary>
        /// An hour of weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a single hour of historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestamp">A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>HistoryPointHourObj</returns>
        HistoryPointHourObj PointsLatitudelongitudeHoursTimestampGet (float? latitude, float? longitude, DateTime? timestamp, List<string> fields, string unitScale = null);

        /// <summary>
        /// An hour of weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a single hour of historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestamp">A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>ApiResponse of HistoryPointHourObj</returns>
        ApiResponse<HistoryPointHourObj> PointsLatitudelongitudeHoursTimestampGetWithHttpInfo (float? latitude, float? longitude, DateTime? timestamp, List<string> fields, string unitScale = null);
        /// <summary>
        /// A contiguous range of hourly weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a contiguous range of hourly historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestampStart">A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.</param>
        /// <param name="timestampEnd">A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>HistoryPointHourObj</returns>
        HistoryPointHourObj PointsLatitudelongitudeHoursTimestampStarttimestampEndGet (float? latitude, float? longitude, DateTime? timestampStart, DateTime? timestampEnd, List<string> fields, string unitScale = null);

        /// <summary>
        /// A contiguous range of hourly weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a contiguous range of hourly historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestampStart">A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.</param>
        /// <param name="timestampEnd">A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>ApiResponse of HistoryPointHourObj</returns>
        ApiResponse<HistoryPointHourObj> PointsLatitudelongitudeHoursTimestampStarttimestampEndGetWithHttpInfo (float? latitude, float? longitude, DateTime? timestampStart, DateTime? timestampEnd, List<string> fields, string unitScale = null);
        #endregion Synchronous Operations
        #region Asynchronous Operations
        /// <summary>
        /// A day of weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a single day of historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="date">An date formatted as a RFC3339 date value.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of HistoryPointDayObj</returns>
        System.Threading.Tasks.Task<HistoryPointDayObj> PointsLatitudelongitudeDaysDateGetAsync (float? latitude, float? longitude, DateTime? date, List<string> fields, string unitScale = null);

        /// <summary>
        /// A day of weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a single day of historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="date">An date formatted as a RFC3339 date value.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of ApiResponse (HistoryPointDayObj)</returns>
        System.Threading.Tasks.Task<ApiResponse<HistoryPointDayObj>> PointsLatitudelongitudeDaysDateGetAsyncWithHttpInfo (float? latitude, float? longitude, DateTime? date, List<string> fields, string unitScale = null);
        /// <summary>
        /// A contiguous range of daily weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a contiguous range of daily historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="dateStart">An date formatted as a RFC3339 date value that must occur temporally before dateEnd.</param>
        /// <param name="dateEnd">An date formatted as a RFC3339 date value that must occur temporally after dateStart.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of HistoryPointDayObj</returns>
        System.Threading.Tasks.Task<HistoryPointDayObj> PointsLatitudelongitudeDaysDateStartdateEndGetAsync (float? latitude, float? longitude, DateTime? dateStart, DateTime? dateEnd, List<string> fields, string unitScale = null);

        /// <summary>
        /// A contiguous range of daily weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a contiguous range of daily historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="dateStart">An date formatted as a RFC3339 date value that must occur temporally before dateEnd.</param>
        /// <param name="dateEnd">An date formatted as a RFC3339 date value that must occur temporally after dateStart.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of ApiResponse (HistoryPointDayObj)</returns>
        System.Threading.Tasks.Task<ApiResponse<HistoryPointDayObj>> PointsLatitudelongitudeDaysDateStartdateEndGetAsyncWithHttpInfo (float? latitude, float? longitude, DateTime? dateStart, DateTime? dateEnd, List<string> fields, string unitScale = null);
        /// <summary>
        /// An hour of weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a single hour of historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestamp">A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of HistoryPointHourObj</returns>
        System.Threading.Tasks.Task<HistoryPointHourObj> PointsLatitudelongitudeHoursTimestampGetAsync (float? latitude, float? longitude, DateTime? timestamp, List<string> fields, string unitScale = null);

        /// <summary>
        /// An hour of weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a single hour of historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestamp">A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of ApiResponse (HistoryPointHourObj)</returns>
        System.Threading.Tasks.Task<ApiResponse<HistoryPointHourObj>> PointsLatitudelongitudeHoursTimestampGetAsyncWithHttpInfo (float? latitude, float? longitude, DateTime? timestamp, List<string> fields, string unitScale = null);
        /// <summary>
        /// A contiguous range of hourly weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a contiguous range of hourly historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestampStart">A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.</param>
        /// <param name="timestampEnd">A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of HistoryPointHourObj</returns>
        System.Threading.Tasks.Task<HistoryPointHourObj> PointsLatitudelongitudeHoursTimestampStarttimestampEndGetAsync (float? latitude, float? longitude, DateTime? timestampStart, DateTime? timestampEnd, List<string> fields, string unitScale = null);

        /// <summary>
        /// A contiguous range of hourly weather history for a Latitude/Longitude point.
        /// </summary>
        /// <remarks>
        /// Returns a contiguous range of hourly historical weather data for a Latitude/Longitude point.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestampStart">A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.</param>
        /// <param name="timestampEnd">A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of ApiResponse (HistoryPointHourObj)</returns>
        System.Threading.Tasks.Task<ApiResponse<HistoryPointHourObj>> PointsLatitudelongitudeHoursTimestampStarttimestampEndGetAsyncWithHttpInfo (float? latitude, float? longitude, DateTime? timestampStart, DateTime? timestampEnd, List<string> fields, string unitScale = null);
        #endregion Asynchronous Operations
    }

    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
        public partial class PointsApi : IPointsApi
    {
        private IO.Swagger.Client.ExceptionFactory _exceptionFactory = (name, response) => null;

        /// <summary>
        /// Initializes a new instance of the <see cref="PointsApi"/> class.
        /// </summary>
        /// <returns></returns>
        public PointsApi(String basePath)
        {
            this.Configuration = new IO.Swagger.Client.Configuration { BasePath = basePath };

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PointsApi"/> class
        /// </summary>
        /// <returns></returns>
        public PointsApi()
        {
            this.Configuration = IO.Swagger.Client.Configuration.Default;

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PointsApi"/> class
        /// using Configuration object
        /// </summary>
        /// <param name="configuration">An instance of Configuration</param>
        /// <returns></returns>
        public PointsApi(IO.Swagger.Client.Configuration configuration = null)
        {
            if (configuration == null) // use the default one in Configuration
                this.Configuration = IO.Swagger.Client.Configuration.Default;
            else
                this.Configuration = configuration;

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Gets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        public String GetBasePath()
        {
            return this.Configuration.ApiClient.RestClient.BaseUrl.ToString();
        }

        /// <summary>
        /// Sets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        [Obsolete("SetBasePath is deprecated, please do 'Configuration.ApiClient = new ApiClient(\"http://new-path\")' instead.")]
        public void SetBasePath(String basePath)
        {
            // do nothing
        }

        /// <summary>
        /// Gets or sets the configuration object
        /// </summary>
        /// <value>An instance of the Configuration</value>
        public IO.Swagger.Client.Configuration Configuration {get; set;}

        /// <summary>
        /// Provides a factory method hook for the creation of exceptions.
        /// </summary>
        public IO.Swagger.Client.ExceptionFactory ExceptionFactory
        {
            get
            {
                if (_exceptionFactory != null && _exceptionFactory.GetInvocationList().Length > 1)
                {
                    throw new InvalidOperationException("Multicast delegate for ExceptionFactory is unsupported.");
                }
                return _exceptionFactory;
            }
            set { _exceptionFactory = value; }
        }

        /// <summary>
        /// Gets the default header.
        /// </summary>
        /// <returns>Dictionary of HTTP header</returns>
        [Obsolete("DefaultHeader is deprecated, please use Configuration.DefaultHeader instead.")]
        public IDictionary<String, String> DefaultHeader()
        {
            return new ReadOnlyDictionary<string, string>(this.Configuration.DefaultHeader);
        }

        /// <summary>
        /// Add default header.
        /// </summary>
        /// <param name="key">Header field name.</param>
        /// <param name="value">Header field value.</param>
        /// <returns></returns>
        [Obsolete("AddDefaultHeader is deprecated, please use Configuration.AddDefaultHeader instead.")]
        public void AddDefaultHeader(string key, string value)
        {
            this.Configuration.AddDefaultHeader(key, value);
        }

        /// <summary>
        /// A day of weather history for a Latitude/Longitude point. Returns a single day of historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="date">An date formatted as a RFC3339 date value.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>HistoryPointDayObj</returns>
        public HistoryPointDayObj PointsLatitudelongitudeDaysDateGet (float? latitude, float? longitude, DateTime? date, List<string> fields, string unitScale = null)
        {
             ApiResponse<HistoryPointDayObj> localVarResponse = PointsLatitudelongitudeDaysDateGetWithHttpInfo(latitude, longitude, date, fields, unitScale);
             return localVarResponse.Data;
        }

        /// <summary>
        /// A day of weather history for a Latitude/Longitude point. Returns a single day of historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="date">An date formatted as a RFC3339 date value.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>ApiResponse of HistoryPointDayObj</returns>
        public ApiResponse< HistoryPointDayObj > PointsLatitudelongitudeDaysDateGetWithHttpInfo (float? latitude, float? longitude, DateTime? date, List<string> fields, string unitScale = null)
        {
            // verify the required parameter 'latitude' is set
            if (latitude == null)
                throw new ApiException(400, "Missing required parameter 'latitude' when calling PointsApi->PointsLatitudelongitudeDaysDateGet");
            // verify the required parameter 'longitude' is set
            if (longitude == null)
                throw new ApiException(400, "Missing required parameter 'longitude' when calling PointsApi->PointsLatitudelongitudeDaysDateGet");
            // verify the required parameter 'date' is set
            if (date == null)
                throw new ApiException(400, "Missing required parameter 'date' when calling PointsApi->PointsLatitudelongitudeDaysDateGet");
            // verify the required parameter 'fields' is set
            if (fields == null)
                throw new ApiException(400, "Missing required parameter 'fields' when calling PointsApi->PointsLatitudelongitudeDaysDateGet");

            var localVarPath = "/points/{latitude},{longitude}/days/{date}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (latitude != null) localVarPathParams.Add("latitude", this.Configuration.ApiClient.ParameterToString(latitude)); // path parameter
            if (longitude != null) localVarPathParams.Add("longitude", this.Configuration.ApiClient.ParameterToString(longitude)); // path parameter
            if (date != null) localVarPathParams.Add("date", this.Configuration.ApiClient.ParameterToString(date)); // path parameter
            if (fields != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("csv", "fields", fields)); // query parameter
            if (unitScale != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "unitScale", unitScale)); // query parameter
            // authentication (ApiKeyAuth) required
            if (!String.IsNullOrEmpty(this.Configuration.GetApiKeyWithPrefix("X-API-KEY")))
            {
                localVarHeaderParams["X-API-KEY"] = this.Configuration.GetApiKeyWithPrefix("X-API-KEY");
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) this.Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PointsLatitudelongitudeDaysDateGet", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<HistoryPointDayObj>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (HistoryPointDayObj) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(HistoryPointDayObj)));
        }

        /// <summary>
        /// A day of weather history for a Latitude/Longitude point. Returns a single day of historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="date">An date formatted as a RFC3339 date value.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of HistoryPointDayObj</returns>
        public async System.Threading.Tasks.Task<HistoryPointDayObj> PointsLatitudelongitudeDaysDateGetAsync (float? latitude, float? longitude, DateTime? date, List<string> fields, string unitScale = null)
        {
             ApiResponse<HistoryPointDayObj> localVarResponse = await PointsLatitudelongitudeDaysDateGetAsyncWithHttpInfo(latitude, longitude, date, fields, unitScale);
             return localVarResponse.Data;

        }

        /// <summary>
        /// A day of weather history for a Latitude/Longitude point. Returns a single day of historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="date">An date formatted as a RFC3339 date value.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of ApiResponse (HistoryPointDayObj)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<HistoryPointDayObj>> PointsLatitudelongitudeDaysDateGetAsyncWithHttpInfo (float? latitude, float? longitude, DateTime? date, List<string> fields, string unitScale = null)
        {
            // verify the required parameter 'latitude' is set
            if (latitude == null)
                throw new ApiException(400, "Missing required parameter 'latitude' when calling PointsApi->PointsLatitudelongitudeDaysDateGet");
            // verify the required parameter 'longitude' is set
            if (longitude == null)
                throw new ApiException(400, "Missing required parameter 'longitude' when calling PointsApi->PointsLatitudelongitudeDaysDateGet");
            // verify the required parameter 'date' is set
            if (date == null)
                throw new ApiException(400, "Missing required parameter 'date' when calling PointsApi->PointsLatitudelongitudeDaysDateGet");
            // verify the required parameter 'fields' is set
            if (fields == null)
                throw new ApiException(400, "Missing required parameter 'fields' when calling PointsApi->PointsLatitudelongitudeDaysDateGet");

            var localVarPath = "/points/{latitude},{longitude}/days/{date}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (latitude != null) localVarPathParams.Add("latitude", this.Configuration.ApiClient.ParameterToString(latitude)); // path parameter
            if (longitude != null) localVarPathParams.Add("longitude", this.Configuration.ApiClient.ParameterToString(longitude)); // path parameter
            if (date != null) localVarPathParams.Add("date", this.Configuration.ApiClient.ParameterToString(date)); // path parameter
            if (fields != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("csv", "fields", fields)); // query parameter
            if (unitScale != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "unitScale", unitScale)); // query parameter
            // authentication (ApiKeyAuth) required
            if (!String.IsNullOrEmpty(this.Configuration.GetApiKeyWithPrefix("X-API-KEY")))
            {
                localVarHeaderParams["X-API-KEY"] = this.Configuration.GetApiKeyWithPrefix("X-API-KEY");
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PointsLatitudelongitudeDaysDateGet", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<HistoryPointDayObj>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (HistoryPointDayObj) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(HistoryPointDayObj)));
        }

        /// <summary>
        /// A contiguous range of daily weather history for a Latitude/Longitude point. Returns a contiguous range of daily historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="dateStart">An date formatted as a RFC3339 date value that must occur temporally before dateEnd.</param>
        /// <param name="dateEnd">An date formatted as a RFC3339 date value that must occur temporally after dateStart.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>HistoryPointDayObj</returns>
        public HistoryPointDayObj PointsLatitudelongitudeDaysDateStartdateEndGet (float? latitude, float? longitude, DateTime? dateStart, DateTime? dateEnd, List<string> fields, string unitScale = null)
        {
             ApiResponse<HistoryPointDayObj> localVarResponse = PointsLatitudelongitudeDaysDateStartdateEndGetWithHttpInfo(latitude, longitude, dateStart, dateEnd, fields, unitScale);
             return localVarResponse.Data;
        }

        /// <summary>
        /// A contiguous range of daily weather history for a Latitude/Longitude point. Returns a contiguous range of daily historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="dateStart">An date formatted as a RFC3339 date value that must occur temporally before dateEnd.</param>
        /// <param name="dateEnd">An date formatted as a RFC3339 date value that must occur temporally after dateStart.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>ApiResponse of HistoryPointDayObj</returns>
        public ApiResponse< HistoryPointDayObj > PointsLatitudelongitudeDaysDateStartdateEndGetWithHttpInfo (float? latitude, float? longitude, DateTime? dateStart, DateTime? dateEnd, List<string> fields, string unitScale = null)
        {
            // verify the required parameter 'latitude' is set
            if (latitude == null)
                throw new ApiException(400, "Missing required parameter 'latitude' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");
            // verify the required parameter 'longitude' is set
            if (longitude == null)
                throw new ApiException(400, "Missing required parameter 'longitude' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");
            // verify the required parameter 'dateStart' is set
            if (dateStart == null)
                throw new ApiException(400, "Missing required parameter 'dateStart' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");
            // verify the required parameter 'dateEnd' is set
            if (dateEnd == null)
                throw new ApiException(400, "Missing required parameter 'dateEnd' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");
            // verify the required parameter 'fields' is set
            if (fields == null)
                throw new ApiException(400, "Missing required parameter 'fields' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");

            var localVarPath = "/points/{latitude},{longitude}/days/{dateStart},{dateEnd}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (latitude != null) localVarPathParams.Add("latitude", this.Configuration.ApiClient.ParameterToString(latitude)); // path parameter
            if (longitude != null) localVarPathParams.Add("longitude", this.Configuration.ApiClient.ParameterToString(longitude)); // path parameter
            if (dateStart != null) localVarPathParams.Add("dateStart", this.Configuration.ApiClient.ParameterToString(dateStart)); // path parameter
            if (dateEnd != null) localVarPathParams.Add("dateEnd", this.Configuration.ApiClient.ParameterToString(dateEnd)); // path parameter
            if (fields != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("csv", "fields", fields)); // query parameter
            if (unitScale != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "unitScale", unitScale)); // query parameter
            // authentication (ApiKeyAuth) required
            if (!String.IsNullOrEmpty(this.Configuration.GetApiKeyWithPrefix("X-API-KEY")))
            {
                localVarHeaderParams["X-API-KEY"] = this.Configuration.GetApiKeyWithPrefix("X-API-KEY");
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) this.Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PointsLatitudelongitudeDaysDateStartdateEndGet", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<HistoryPointDayObj>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (HistoryPointDayObj) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(HistoryPointDayObj)));
        }

        /// <summary>
        /// A contiguous range of daily weather history for a Latitude/Longitude point. Returns a contiguous range of daily historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="dateStart">An date formatted as a RFC3339 date value that must occur temporally before dateEnd.</param>
        /// <param name="dateEnd">An date formatted as a RFC3339 date value that must occur temporally after dateStart.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of HistoryPointDayObj</returns>
        public async System.Threading.Tasks.Task<HistoryPointDayObj> PointsLatitudelongitudeDaysDateStartdateEndGetAsync (float? latitude, float? longitude, DateTime? dateStart, DateTime? dateEnd, List<string> fields, string unitScale = null)
        {
             ApiResponse<HistoryPointDayObj> localVarResponse = await PointsLatitudelongitudeDaysDateStartdateEndGetAsyncWithHttpInfo(latitude, longitude, dateStart, dateEnd, fields, unitScale);
             return localVarResponse.Data;

        }

        /// <summary>
        /// A contiguous range of daily weather history for a Latitude/Longitude point. Returns a contiguous range of daily historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="dateStart">An date formatted as a RFC3339 date value that must occur temporally before dateEnd.</param>
        /// <param name="dateEnd">An date formatted as a RFC3339 date value that must occur temporally after dateStart.</param>
        /// <param name="fields">A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of ApiResponse (HistoryPointDayObj)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<HistoryPointDayObj>> PointsLatitudelongitudeDaysDateStartdateEndGetAsyncWithHttpInfo (float? latitude, float? longitude, DateTime? dateStart, DateTime? dateEnd, List<string> fields, string unitScale = null)
        {
            // verify the required parameter 'latitude' is set
            if (latitude == null)
                throw new ApiException(400, "Missing required parameter 'latitude' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");
            // verify the required parameter 'longitude' is set
            if (longitude == null)
                throw new ApiException(400, "Missing required parameter 'longitude' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");
            // verify the required parameter 'dateStart' is set
            if (dateStart == null)
                throw new ApiException(400, "Missing required parameter 'dateStart' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");
            // verify the required parameter 'dateEnd' is set
            if (dateEnd == null)
                throw new ApiException(400, "Missing required parameter 'dateEnd' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");
            // verify the required parameter 'fields' is set
            if (fields == null)
                throw new ApiException(400, "Missing required parameter 'fields' when calling PointsApi->PointsLatitudelongitudeDaysDateStartdateEndGet");

            var localVarPath = "/points/{latitude},{longitude}/days/{dateStart},{dateEnd}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (latitude != null) localVarPathParams.Add("latitude", this.Configuration.ApiClient.ParameterToString(latitude)); // path parameter
            if (longitude != null) localVarPathParams.Add("longitude", this.Configuration.ApiClient.ParameterToString(longitude)); // path parameter
            if (dateStart != null) localVarPathParams.Add("dateStart", this.Configuration.ApiClient.ParameterToString(dateStart)); // path parameter
            if (dateEnd != null) localVarPathParams.Add("dateEnd", this.Configuration.ApiClient.ParameterToString(dateEnd)); // path parameter
            if (fields != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("csv", "fields", fields)); // query parameter
            if (unitScale != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "unitScale", unitScale)); // query parameter
            // authentication (ApiKeyAuth) required
            if (!String.IsNullOrEmpty(this.Configuration.GetApiKeyWithPrefix("X-API-KEY")))
            {
                localVarHeaderParams["X-API-KEY"] = this.Configuration.GetApiKeyWithPrefix("X-API-KEY");
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PointsLatitudelongitudeDaysDateStartdateEndGet", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<HistoryPointDayObj>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (HistoryPointDayObj) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(HistoryPointDayObj)));
        }

        /// <summary>
        /// An hour of weather history for a Latitude/Longitude point. Returns a single hour of historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestamp">A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>HistoryPointHourObj</returns>
        public HistoryPointHourObj PointsLatitudelongitudeHoursTimestampGet (float? latitude, float? longitude, DateTime? timestamp, List<string> fields, string unitScale = null)
        {
             ApiResponse<HistoryPointHourObj> localVarResponse = PointsLatitudelongitudeHoursTimestampGetWithHttpInfo(latitude, longitude, timestamp, fields, unitScale);
             return localVarResponse.Data;
        }

        /// <summary>
        /// An hour of weather history for a Latitude/Longitude point. Returns a single hour of historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestamp">A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>ApiResponse of HistoryPointHourObj</returns>
        public ApiResponse< HistoryPointHourObj > PointsLatitudelongitudeHoursTimestampGetWithHttpInfo (float? latitude, float? longitude, DateTime? timestamp, List<string> fields, string unitScale = null)
        {
            // verify the required parameter 'latitude' is set
            if (latitude == null)
                throw new ApiException(400, "Missing required parameter 'latitude' when calling PointsApi->PointsLatitudelongitudeHoursTimestampGet");
            // verify the required parameter 'longitude' is set
            if (longitude == null)
                throw new ApiException(400, "Missing required parameter 'longitude' when calling PointsApi->PointsLatitudelongitudeHoursTimestampGet");
            // verify the required parameter 'timestamp' is set
            if (timestamp == null)
                throw new ApiException(400, "Missing required parameter 'timestamp' when calling PointsApi->PointsLatitudelongitudeHoursTimestampGet");
            // verify the required parameter 'fields' is set
            if (fields == null)
                throw new ApiException(400, "Missing required parameter 'fields' when calling PointsApi->PointsLatitudelongitudeHoursTimestampGet");

            var localVarPath = "/points/{latitude},{longitude}/hours/{timestamp}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (latitude != null) localVarPathParams.Add("latitude", this.Configuration.ApiClient.ParameterToString(latitude)); // path parameter
            if (longitude != null) localVarPathParams.Add("longitude", this.Configuration.ApiClient.ParameterToString(longitude)); // path parameter
            if (timestamp != null) localVarPathParams.Add("timestamp", this.Configuration.ApiClient.ParameterToString(timestamp)); // path parameter
            if (fields != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("csv", "fields", fields)); // query parameter
            if (unitScale != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "unitScale", unitScale)); // query parameter
            // authentication (ApiKeyAuth) required
            if (!String.IsNullOrEmpty(this.Configuration.GetApiKeyWithPrefix("X-API-KEY")))
            {
                localVarHeaderParams["X-API-KEY"] = this.Configuration.GetApiKeyWithPrefix("X-API-KEY");
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) this.Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PointsLatitudelongitudeHoursTimestampGet", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<HistoryPointHourObj>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (HistoryPointHourObj) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(HistoryPointHourObj)));
        }

        /// <summary>
        /// An hour of weather history for a Latitude/Longitude point. Returns a single hour of historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestamp">A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of HistoryPointHourObj</returns>
        public async System.Threading.Tasks.Task<HistoryPointHourObj> PointsLatitudelongitudeHoursTimestampGetAsync (float? latitude, float? longitude, DateTime? timestamp, List<string> fields, string unitScale = null)
        {
             ApiResponse<HistoryPointHourObj> localVarResponse = await PointsLatitudelongitudeHoursTimestampGetAsyncWithHttpInfo(latitude, longitude, timestamp, fields, unitScale);
             return localVarResponse.Data;

        }

        /// <summary>
        /// An hour of weather history for a Latitude/Longitude point. Returns a single hour of historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestamp">A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of ApiResponse (HistoryPointHourObj)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<HistoryPointHourObj>> PointsLatitudelongitudeHoursTimestampGetAsyncWithHttpInfo (float? latitude, float? longitude, DateTime? timestamp, List<string> fields, string unitScale = null)
        {
            // verify the required parameter 'latitude' is set
            if (latitude == null)
                throw new ApiException(400, "Missing required parameter 'latitude' when calling PointsApi->PointsLatitudelongitudeHoursTimestampGet");
            // verify the required parameter 'longitude' is set
            if (longitude == null)
                throw new ApiException(400, "Missing required parameter 'longitude' when calling PointsApi->PointsLatitudelongitudeHoursTimestampGet");
            // verify the required parameter 'timestamp' is set
            if (timestamp == null)
                throw new ApiException(400, "Missing required parameter 'timestamp' when calling PointsApi->PointsLatitudelongitudeHoursTimestampGet");
            // verify the required parameter 'fields' is set
            if (fields == null)
                throw new ApiException(400, "Missing required parameter 'fields' when calling PointsApi->PointsLatitudelongitudeHoursTimestampGet");

            var localVarPath = "/points/{latitude},{longitude}/hours/{timestamp}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (latitude != null) localVarPathParams.Add("latitude", this.Configuration.ApiClient.ParameterToString(latitude)); // path parameter
            if (longitude != null) localVarPathParams.Add("longitude", this.Configuration.ApiClient.ParameterToString(longitude)); // path parameter
            if (timestamp != null) localVarPathParams.Add("timestamp", this.Configuration.ApiClient.ParameterToString(timestamp)); // path parameter
            if (fields != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("csv", "fields", fields)); // query parameter
            if (unitScale != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "unitScale", unitScale)); // query parameter
            // authentication (ApiKeyAuth) required
            if (!String.IsNullOrEmpty(this.Configuration.GetApiKeyWithPrefix("X-API-KEY")))
            {
                localVarHeaderParams["X-API-KEY"] = this.Configuration.GetApiKeyWithPrefix("X-API-KEY");
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PointsLatitudelongitudeHoursTimestampGet", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<HistoryPointHourObj>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (HistoryPointHourObj) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(HistoryPointHourObj)));
        }

        /// <summary>
        /// A contiguous range of hourly weather history for a Latitude/Longitude point. Returns a contiguous range of hourly historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestampStart">A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.</param>
        /// <param name="timestampEnd">A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>HistoryPointHourObj</returns>
        public HistoryPointHourObj PointsLatitudelongitudeHoursTimestampStarttimestampEndGet (float? latitude, float? longitude, DateTime? timestampStart, DateTime? timestampEnd, List<string> fields, string unitScale = null)
        {
             ApiResponse<HistoryPointHourObj> localVarResponse = PointsLatitudelongitudeHoursTimestampStarttimestampEndGetWithHttpInfo(latitude, longitude, timestampStart, timestampEnd, fields, unitScale);
             return localVarResponse.Data;
        }

        /// <summary>
        /// A contiguous range of hourly weather history for a Latitude/Longitude point. Returns a contiguous range of hourly historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestampStart">A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.</param>
        /// <param name="timestampEnd">A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>ApiResponse of HistoryPointHourObj</returns>
        public ApiResponse< HistoryPointHourObj > PointsLatitudelongitudeHoursTimestampStarttimestampEndGetWithHttpInfo (float? latitude, float? longitude, DateTime? timestampStart, DateTime? timestampEnd, List<string> fields, string unitScale = null)
        {
            // verify the required parameter 'latitude' is set
            if (latitude == null)
                throw new ApiException(400, "Missing required parameter 'latitude' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
            // verify the required parameter 'longitude' is set
            if (longitude == null)
                throw new ApiException(400, "Missing required parameter 'longitude' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
            // verify the required parameter 'timestampStart' is set
            if (timestampStart == null)
                throw new ApiException(400, "Missing required parameter 'timestampStart' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
            // verify the required parameter 'timestampEnd' is set
            if (timestampEnd == null)
                throw new ApiException(400, "Missing required parameter 'timestampEnd' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
            // verify the required parameter 'fields' is set
            if (fields == null)
                throw new ApiException(400, "Missing required parameter 'fields' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");

            var localVarPath = "/points/{latitude},{longitude}/hours/{timestampStart},{timestampEnd}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (latitude != null) localVarPathParams.Add("latitude", this.Configuration.ApiClient.ParameterToString(latitude)); // path parameter
            if (longitude != null) localVarPathParams.Add("longitude", this.Configuration.ApiClient.ParameterToString(longitude)); // path parameter
            if (timestampStart != null) localVarPathParams.Add("timestampStart", this.Configuration.ApiClient.ParameterToString(timestampStart)); // path parameter
            if (timestampEnd != null) localVarPathParams.Add("timestampEnd", this.Configuration.ApiClient.ParameterToString(timestampEnd)); // path parameter
            if (fields != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("csv", "fields", fields)); // query parameter
            if (unitScale != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "unitScale", unitScale)); // query parameter
            // authentication (ApiKeyAuth) required
            if (!String.IsNullOrEmpty(this.Configuration.GetApiKeyWithPrefix("X-API-KEY")))
            {
                localVarHeaderParams["X-API-KEY"] = this.Configuration.GetApiKeyWithPrefix("X-API-KEY");
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) this.Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PointsLatitudelongitudeHoursTimestampStarttimestampEndGet", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<HistoryPointHourObj>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (HistoryPointHourObj) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(HistoryPointHourObj)));
        }

        /// <summary>
        /// A contiguous range of hourly weather history for a Latitude/Longitude point. Returns a contiguous range of hourly historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestampStart">A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.</param>
        /// <param name="timestampEnd">A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of HistoryPointHourObj</returns>
        public async System.Threading.Tasks.Task<HistoryPointHourObj> PointsLatitudelongitudeHoursTimestampStarttimestampEndGetAsync (float? latitude, float? longitude, DateTime? timestampStart, DateTime? timestampEnd, List<string> fields, string unitScale = null)
        {
             ApiResponse<HistoryPointHourObj> localVarResponse = await PointsLatitudelongitudeHoursTimestampStarttimestampEndGetAsyncWithHttpInfo(latitude, longitude, timestampStart, timestampEnd, fields, unitScale);
             return localVarResponse.Data;

        }

        /// <summary>
        /// A contiguous range of hourly weather history for a Latitude/Longitude point. Returns a contiguous range of hourly historical weather data for a Latitude/Longitude point.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="latitude">A latitude value between -90° and 90°.</param>
        /// <param name="longitude">A longitude value between -180° (West) and 180° (East).</param>
        /// <param name="timestampStart">A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.</param>
        /// <param name="timestampEnd">A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.</param>
        /// <param name="fields">A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* </param>
        /// <param name="unitScale">The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)</param>
        /// <returns>Task of ApiResponse (HistoryPointHourObj)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<HistoryPointHourObj>> PointsLatitudelongitudeHoursTimestampStarttimestampEndGetAsyncWithHttpInfo (float? latitude, float? longitude, DateTime? timestampStart, DateTime? timestampEnd, List<string> fields, string unitScale = null)
        {
            // verify the required parameter 'latitude' is set
            if (latitude == null)
                throw new ApiException(400, "Missing required parameter 'latitude' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
            // verify the required parameter 'longitude' is set
            if (longitude == null)
                throw new ApiException(400, "Missing required parameter 'longitude' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
            // verify the required parameter 'timestampStart' is set
            if (timestampStart == null)
                throw new ApiException(400, "Missing required parameter 'timestampStart' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
            // verify the required parameter 'timestampEnd' is set
            if (timestampEnd == null)
                throw new ApiException(400, "Missing required parameter 'timestampEnd' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");
            // verify the required parameter 'fields' is set
            if (fields == null)
                throw new ApiException(400, "Missing required parameter 'fields' when calling PointsApi->PointsLatitudelongitudeHoursTimestampStarttimestampEndGet");

            var localVarPath = "/points/{latitude},{longitude}/hours/{timestampStart},{timestampEnd}";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (latitude != null) localVarPathParams.Add("latitude", this.Configuration.ApiClient.ParameterToString(latitude)); // path parameter
            if (longitude != null) localVarPathParams.Add("longitude", this.Configuration.ApiClient.ParameterToString(longitude)); // path parameter
            if (timestampStart != null) localVarPathParams.Add("timestampStart", this.Configuration.ApiClient.ParameterToString(timestampStart)); // path parameter
            if (timestampEnd != null) localVarPathParams.Add("timestampEnd", this.Configuration.ApiClient.ParameterToString(timestampEnd)); // path parameter
            if (fields != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("csv", "fields", fields)); // query parameter
            if (unitScale != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "unitScale", unitScale)); // query parameter
            // authentication (ApiKeyAuth) required
            if (!String.IsNullOrEmpty(this.Configuration.GetApiKeyWithPrefix("X-API-KEY")))
            {
                localVarHeaderParams["X-API-KEY"] = this.Configuration.GetApiKeyWithPrefix("X-API-KEY");
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("PointsLatitudelongitudeHoursTimestampStarttimestampEndGet", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<HistoryPointHourObj>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (HistoryPointHourObj) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(HistoryPointHourObj)));
        }

    }
}
