/* 
 * Weather Source History API
 *
 * Daily and hourly historical weather data for latitude/longitude points, Zip/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source History API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */
using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// An hourly weather history object.
    /// </summary>
    [DataContract]
        public partial class HistoryHourObj :  IEquatable<HistoryHourObj>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="HistoryHourObj" /> class.
        /// </summary>
        /// <param name="timestamp">timestamp (required).</param>
        /// <param name="provisionalFlag">provisionalFlag.</param>
        /// <param name="cldCvr">cldCvr.</param>
        /// <param name="dewPt">dewPt.</param>
        /// <param name="feelsLike">feelsLike.</param>
        /// <param name="heatIndex">heatIndex.</param>
        /// <param name="mslPres">mslPres.</param>
        /// <param name="precip">precip.</param>
        /// <param name="presTend">presTend.</param>
        /// <param name="radSolar">radSolar.</param>
        /// <param name="relHum">relHum.</param>
        /// <param name="sfcPres">sfcPres.</param>
        /// <param name="snowfall">snowfall.</param>
        /// <param name="spcHum">spcHum.</param>
        /// <param name="temp">temp.</param>
        /// <param name="vis">vis.</param>
        /// <param name="wetBulb">wetBulb.</param>
        /// <param name="windChill">windChill.</param>
        /// <param name="windDir">windDir.</param>
        /// <param name="windDir80m">windDir80m.</param>
        /// <param name="windDir100m">windDir100m.</param>
        /// <param name="windSpd">windSpd.</param>
        /// <param name="windSpd80m">windSpd80m.</param>
        /// <param name="windSpd100m">windSpd100m.</param>
        /// <param name="freezingRainFlag">freezingRainFlag.</param>
        /// <param name="icePelletsFlag">icePelletsFlag.</param>
        /// <param name="rainFlag">rainFlag.</param>
        /// <param name="snowFlag">snowFlag.</param>
        public HistoryHourObj(Timestamp timestamp = default(Timestamp), ProvisionalFlag provisionalFlag = default(ProvisionalFlag), CldCvr cldCvr = default(CldCvr), DewPt dewPt = default(DewPt), FeelsLike feelsLike = default(FeelsLike), HeatIndex heatIndex = default(HeatIndex), MslPres mslPres = default(MslPres), Precip precip = default(Precip), PresTend presTend = default(PresTend), RadSolar radSolar = default(RadSolar), RelHum relHum = default(RelHum), SfcPres sfcPres = default(SfcPres), Snowfall snowfall = default(Snowfall), SpcHum spcHum = default(SpcHum), Temp temp = default(Temp), Vis vis = default(Vis), WetBulb wetBulb = default(WetBulb), WindChill windChill = default(WindChill), WindDir windDir = default(WindDir), WindDir80m windDir80m = default(WindDir80m), WindDir100m windDir100m = default(WindDir100m), WindSpd windSpd = default(WindSpd), WindSpd80m windSpd80m = default(WindSpd80m), WindSpd100m windSpd100m = default(WindSpd100m), FreezingRainFlag freezingRainFlag = default(FreezingRainFlag), IcePelletsFlag icePelletsFlag = default(IcePelletsFlag), RainFlag rainFlag = default(RainFlag), SnowFlag snowFlag = default(SnowFlag))
        {
            // to ensure "timestamp" is required (not null)
            if (timestamp == null)
            {
                throw new InvalidDataException("timestamp is a required property for HistoryHourObj and cannot be null");
            }
            else
            {
                this.Timestamp = timestamp;
            }
            this.ProvisionalFlag = provisionalFlag;
            this.CldCvr = cldCvr;
            this.DewPt = dewPt;
            this.FeelsLike = feelsLike;
            this.HeatIndex = heatIndex;
            this.MslPres = mslPres;
            this.Precip = precip;
            this.PresTend = presTend;
            this.RadSolar = radSolar;
            this.RelHum = relHum;
            this.SfcPres = sfcPres;
            this.Snowfall = snowfall;
            this.SpcHum = spcHum;
            this.Temp = temp;
            this.Vis = vis;
            this.WetBulb = wetBulb;
            this.WindChill = windChill;
            this.WindDir = windDir;
            this.WindDir80m = windDir80m;
            this.WindDir100m = windDir100m;
            this.WindSpd = windSpd;
            this.WindSpd80m = windSpd80m;
            this.WindSpd100m = windSpd100m;
            this.FreezingRainFlag = freezingRainFlag;
            this.IcePelletsFlag = icePelletsFlag;
            this.RainFlag = rainFlag;
            this.SnowFlag = snowFlag;
        }
        
        /// <summary>
        /// Gets or Sets Timestamp
        /// </summary>
        [DataMember(Name="timestamp", EmitDefaultValue=false)]
        public Timestamp Timestamp { get; set; }

        /// <summary>
        /// Gets or Sets ProvisionalFlag
        /// </summary>
        [DataMember(Name="provisionalFlag", EmitDefaultValue=false)]
        public ProvisionalFlag ProvisionalFlag { get; set; }

        /// <summary>
        /// Gets or Sets CldCvr
        /// </summary>
        [DataMember(Name="cldCvr", EmitDefaultValue=false)]
        public CldCvr CldCvr { get; set; }

        /// <summary>
        /// Gets or Sets DewPt
        /// </summary>
        [DataMember(Name="dewPt", EmitDefaultValue=false)]
        public DewPt DewPt { get; set; }

        /// <summary>
        /// Gets or Sets FeelsLike
        /// </summary>
        [DataMember(Name="feelsLike", EmitDefaultValue=false)]
        public FeelsLike FeelsLike { get; set; }

        /// <summary>
        /// Gets or Sets HeatIndex
        /// </summary>
        [DataMember(Name="heatIndex", EmitDefaultValue=false)]
        public HeatIndex HeatIndex { get; set; }

        /// <summary>
        /// Gets or Sets MslPres
        /// </summary>
        [DataMember(Name="mslPres", EmitDefaultValue=false)]
        public MslPres MslPres { get; set; }

        /// <summary>
        /// Gets or Sets Precip
        /// </summary>
        [DataMember(Name="precip", EmitDefaultValue=false)]
        public Precip Precip { get; set; }

        /// <summary>
        /// Gets or Sets PresTend
        /// </summary>
        [DataMember(Name="presTend", EmitDefaultValue=false)]
        public PresTend PresTend { get; set; }

        /// <summary>
        /// Gets or Sets RadSolar
        /// </summary>
        [DataMember(Name="radSolar", EmitDefaultValue=false)]
        public RadSolar RadSolar { get; set; }

        /// <summary>
        /// Gets or Sets RelHum
        /// </summary>
        [DataMember(Name="relHum", EmitDefaultValue=false)]
        public RelHum RelHum { get; set; }

        /// <summary>
        /// Gets or Sets SfcPres
        /// </summary>
        [DataMember(Name="sfcPres", EmitDefaultValue=false)]
        public SfcPres SfcPres { get; set; }

        /// <summary>
        /// Gets or Sets Snowfall
        /// </summary>
        [DataMember(Name="snowfall", EmitDefaultValue=false)]
        public Snowfall Snowfall { get; set; }

        /// <summary>
        /// Gets or Sets SpcHum
        /// </summary>
        [DataMember(Name="spcHum", EmitDefaultValue=false)]
        public SpcHum SpcHum { get; set; }

        /// <summary>
        /// Gets or Sets Temp
        /// </summary>
        [DataMember(Name="temp", EmitDefaultValue=false)]
        public Temp Temp { get; set; }

        /// <summary>
        /// Gets or Sets Vis
        /// </summary>
        [DataMember(Name="vis", EmitDefaultValue=false)]
        public Vis Vis { get; set; }

        /// <summary>
        /// Gets or Sets WetBulb
        /// </summary>
        [DataMember(Name="wetBulb", EmitDefaultValue=false)]
        public WetBulb WetBulb { get; set; }

        /// <summary>
        /// Gets or Sets WindChill
        /// </summary>
        [DataMember(Name="windChill", EmitDefaultValue=false)]
        public WindChill WindChill { get; set; }

        /// <summary>
        /// Gets or Sets WindDir
        /// </summary>
        [DataMember(Name="windDir", EmitDefaultValue=false)]
        public WindDir WindDir { get; set; }

        /// <summary>
        /// Gets or Sets WindDir80m
        /// </summary>
        [DataMember(Name="windDir80m", EmitDefaultValue=false)]
        public WindDir80m WindDir80m { get; set; }

        /// <summary>
        /// Gets or Sets WindDir100m
        /// </summary>
        [DataMember(Name="windDir100m", EmitDefaultValue=false)]
        public WindDir100m WindDir100m { get; set; }

        /// <summary>
        /// Gets or Sets WindSpd
        /// </summary>
        [DataMember(Name="windSpd", EmitDefaultValue=false)]
        public WindSpd WindSpd { get; set; }

        /// <summary>
        /// Gets or Sets WindSpd80m
        /// </summary>
        [DataMember(Name="windSpd80m", EmitDefaultValue=false)]
        public WindSpd80m WindSpd80m { get; set; }

        /// <summary>
        /// Gets or Sets WindSpd100m
        /// </summary>
        [DataMember(Name="windSpd100m", EmitDefaultValue=false)]
        public WindSpd100m WindSpd100m { get; set; }

        /// <summary>
        /// Gets or Sets FreezingRainFlag
        /// </summary>
        [DataMember(Name="freezingRainFlag", EmitDefaultValue=false)]
        public FreezingRainFlag FreezingRainFlag { get; set; }

        /// <summary>
        /// Gets or Sets IcePelletsFlag
        /// </summary>
        [DataMember(Name="icePelletsFlag", EmitDefaultValue=false)]
        public IcePelletsFlag IcePelletsFlag { get; set; }

        /// <summary>
        /// Gets or Sets RainFlag
        /// </summary>
        [DataMember(Name="rainFlag", EmitDefaultValue=false)]
        public RainFlag RainFlag { get; set; }

        /// <summary>
        /// Gets or Sets SnowFlag
        /// </summary>
        [DataMember(Name="snowFlag", EmitDefaultValue=false)]
        public SnowFlag SnowFlag { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class HistoryHourObj {\n");
            sb.Append("  Timestamp: ").Append(Timestamp).Append("\n");
            sb.Append("  ProvisionalFlag: ").Append(ProvisionalFlag).Append("\n");
            sb.Append("  CldCvr: ").Append(CldCvr).Append("\n");
            sb.Append("  DewPt: ").Append(DewPt).Append("\n");
            sb.Append("  FeelsLike: ").Append(FeelsLike).Append("\n");
            sb.Append("  HeatIndex: ").Append(HeatIndex).Append("\n");
            sb.Append("  MslPres: ").Append(MslPres).Append("\n");
            sb.Append("  Precip: ").Append(Precip).Append("\n");
            sb.Append("  PresTend: ").Append(PresTend).Append("\n");
            sb.Append("  RadSolar: ").Append(RadSolar).Append("\n");
            sb.Append("  RelHum: ").Append(RelHum).Append("\n");
            sb.Append("  SfcPres: ").Append(SfcPres).Append("\n");
            sb.Append("  Snowfall: ").Append(Snowfall).Append("\n");
            sb.Append("  SpcHum: ").Append(SpcHum).Append("\n");
            sb.Append("  Temp: ").Append(Temp).Append("\n");
            sb.Append("  Vis: ").Append(Vis).Append("\n");
            sb.Append("  WetBulb: ").Append(WetBulb).Append("\n");
            sb.Append("  WindChill: ").Append(WindChill).Append("\n");
            sb.Append("  WindDir: ").Append(WindDir).Append("\n");
            sb.Append("  WindDir80m: ").Append(WindDir80m).Append("\n");
            sb.Append("  WindDir100m: ").Append(WindDir100m).Append("\n");
            sb.Append("  WindSpd: ").Append(WindSpd).Append("\n");
            sb.Append("  WindSpd80m: ").Append(WindSpd80m).Append("\n");
            sb.Append("  WindSpd100m: ").Append(WindSpd100m).Append("\n");
            sb.Append("  FreezingRainFlag: ").Append(FreezingRainFlag).Append("\n");
            sb.Append("  IcePelletsFlag: ").Append(IcePelletsFlag).Append("\n");
            sb.Append("  RainFlag: ").Append(RainFlag).Append("\n");
            sb.Append("  SnowFlag: ").Append(SnowFlag).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as HistoryHourObj);
        }

        /// <summary>
        /// Returns true if HistoryHourObj instances are equal
        /// </summary>
        /// <param name="input">Instance of HistoryHourObj to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(HistoryHourObj input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Timestamp == input.Timestamp ||
                    (this.Timestamp != null &&
                    this.Timestamp.Equals(input.Timestamp))
                ) && 
                (
                    this.ProvisionalFlag == input.ProvisionalFlag ||
                    (this.ProvisionalFlag != null &&
                    this.ProvisionalFlag.Equals(input.ProvisionalFlag))
                ) && 
                (
                    this.CldCvr == input.CldCvr ||
                    (this.CldCvr != null &&
                    this.CldCvr.Equals(input.CldCvr))
                ) && 
                (
                    this.DewPt == input.DewPt ||
                    (this.DewPt != null &&
                    this.DewPt.Equals(input.DewPt))
                ) && 
                (
                    this.FeelsLike == input.FeelsLike ||
                    (this.FeelsLike != null &&
                    this.FeelsLike.Equals(input.FeelsLike))
                ) && 
                (
                    this.HeatIndex == input.HeatIndex ||
                    (this.HeatIndex != null &&
                    this.HeatIndex.Equals(input.HeatIndex))
                ) && 
                (
                    this.MslPres == input.MslPres ||
                    (this.MslPres != null &&
                    this.MslPres.Equals(input.MslPres))
                ) && 
                (
                    this.Precip == input.Precip ||
                    (this.Precip != null &&
                    this.Precip.Equals(input.Precip))
                ) && 
                (
                    this.PresTend == input.PresTend ||
                    (this.PresTend != null &&
                    this.PresTend.Equals(input.PresTend))
                ) && 
                (
                    this.RadSolar == input.RadSolar ||
                    (this.RadSolar != null &&
                    this.RadSolar.Equals(input.RadSolar))
                ) && 
                (
                    this.RelHum == input.RelHum ||
                    (this.RelHum != null &&
                    this.RelHum.Equals(input.RelHum))
                ) && 
                (
                    this.SfcPres == input.SfcPres ||
                    (this.SfcPres != null &&
                    this.SfcPres.Equals(input.SfcPres))
                ) && 
                (
                    this.Snowfall == input.Snowfall ||
                    (this.Snowfall != null &&
                    this.Snowfall.Equals(input.Snowfall))
                ) && 
                (
                    this.SpcHum == input.SpcHum ||
                    (this.SpcHum != null &&
                    this.SpcHum.Equals(input.SpcHum))
                ) && 
                (
                    this.Temp == input.Temp ||
                    (this.Temp != null &&
                    this.Temp.Equals(input.Temp))
                ) && 
                (
                    this.Vis == input.Vis ||
                    (this.Vis != null &&
                    this.Vis.Equals(input.Vis))
                ) && 
                (
                    this.WetBulb == input.WetBulb ||
                    (this.WetBulb != null &&
                    this.WetBulb.Equals(input.WetBulb))
                ) && 
                (
                    this.WindChill == input.WindChill ||
                    (this.WindChill != null &&
                    this.WindChill.Equals(input.WindChill))
                ) && 
                (
                    this.WindDir == input.WindDir ||
                    (this.WindDir != null &&
                    this.WindDir.Equals(input.WindDir))
                ) && 
                (
                    this.WindDir80m == input.WindDir80m ||
                    (this.WindDir80m != null &&
                    this.WindDir80m.Equals(input.WindDir80m))
                ) && 
                (
                    this.WindDir100m == input.WindDir100m ||
                    (this.WindDir100m != null &&
                    this.WindDir100m.Equals(input.WindDir100m))
                ) && 
                (
                    this.WindSpd == input.WindSpd ||
                    (this.WindSpd != null &&
                    this.WindSpd.Equals(input.WindSpd))
                ) && 
                (
                    this.WindSpd80m == input.WindSpd80m ||
                    (this.WindSpd80m != null &&
                    this.WindSpd80m.Equals(input.WindSpd80m))
                ) && 
                (
                    this.WindSpd100m == input.WindSpd100m ||
                    (this.WindSpd100m != null &&
                    this.WindSpd100m.Equals(input.WindSpd100m))
                ) && 
                (
                    this.FreezingRainFlag == input.FreezingRainFlag ||
                    (this.FreezingRainFlag != null &&
                    this.FreezingRainFlag.Equals(input.FreezingRainFlag))
                ) && 
                (
                    this.IcePelletsFlag == input.IcePelletsFlag ||
                    (this.IcePelletsFlag != null &&
                    this.IcePelletsFlag.Equals(input.IcePelletsFlag))
                ) && 
                (
                    this.RainFlag == input.RainFlag ||
                    (this.RainFlag != null &&
                    this.RainFlag.Equals(input.RainFlag))
                ) && 
                (
                    this.SnowFlag == input.SnowFlag ||
                    (this.SnowFlag != null &&
                    this.SnowFlag.Equals(input.SnowFlag))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Timestamp != null)
                    hashCode = hashCode * 59 + this.Timestamp.GetHashCode();
                if (this.ProvisionalFlag != null)
                    hashCode = hashCode * 59 + this.ProvisionalFlag.GetHashCode();
                if (this.CldCvr != null)
                    hashCode = hashCode * 59 + this.CldCvr.GetHashCode();
                if (this.DewPt != null)
                    hashCode = hashCode * 59 + this.DewPt.GetHashCode();
                if (this.FeelsLike != null)
                    hashCode = hashCode * 59 + this.FeelsLike.GetHashCode();
                if (this.HeatIndex != null)
                    hashCode = hashCode * 59 + this.HeatIndex.GetHashCode();
                if (this.MslPres != null)
                    hashCode = hashCode * 59 + this.MslPres.GetHashCode();
                if (this.Precip != null)
                    hashCode = hashCode * 59 + this.Precip.GetHashCode();
                if (this.PresTend != null)
                    hashCode = hashCode * 59 + this.PresTend.GetHashCode();
                if (this.RadSolar != null)
                    hashCode = hashCode * 59 + this.RadSolar.GetHashCode();
                if (this.RelHum != null)
                    hashCode = hashCode * 59 + this.RelHum.GetHashCode();
                if (this.SfcPres != null)
                    hashCode = hashCode * 59 + this.SfcPres.GetHashCode();
                if (this.Snowfall != null)
                    hashCode = hashCode * 59 + this.Snowfall.GetHashCode();
                if (this.SpcHum != null)
                    hashCode = hashCode * 59 + this.SpcHum.GetHashCode();
                if (this.Temp != null)
                    hashCode = hashCode * 59 + this.Temp.GetHashCode();
                if (this.Vis != null)
                    hashCode = hashCode * 59 + this.Vis.GetHashCode();
                if (this.WetBulb != null)
                    hashCode = hashCode * 59 + this.WetBulb.GetHashCode();
                if (this.WindChill != null)
                    hashCode = hashCode * 59 + this.WindChill.GetHashCode();
                if (this.WindDir != null)
                    hashCode = hashCode * 59 + this.WindDir.GetHashCode();
                if (this.WindDir80m != null)
                    hashCode = hashCode * 59 + this.WindDir80m.GetHashCode();
                if (this.WindDir100m != null)
                    hashCode = hashCode * 59 + this.WindDir100m.GetHashCode();
                if (this.WindSpd != null)
                    hashCode = hashCode * 59 + this.WindSpd.GetHashCode();
                if (this.WindSpd80m != null)
                    hashCode = hashCode * 59 + this.WindSpd80m.GetHashCode();
                if (this.WindSpd100m != null)
                    hashCode = hashCode * 59 + this.WindSpd100m.GetHashCode();
                if (this.FreezingRainFlag != null)
                    hashCode = hashCode * 59 + this.FreezingRainFlag.GetHashCode();
                if (this.IcePelletsFlag != null)
                    hashCode = hashCode * 59 + this.IcePelletsFlag.GetHashCode();
                if (this.RainFlag != null)
                    hashCode = hashCode * 59 + this.RainFlag.GetHashCode();
                if (this.SnowFlag != null)
                    hashCode = hashCode * 59 + this.SnowFlag.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }
}
