/* 
 * Weather Source History API
 *
 * Daily and hourly historical weather data for latitude/longitude points, Zip/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source History API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */
using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// Daily Latitude/Longitude historical weather object
    /// </summary>
    [DataContract]
        public partial class HistoryPointDayObj :  IEquatable<HistoryPointDayObj>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="HistoryPointDayObj" /> class.
        /// </summary>
        /// <param name="location">location (required).</param>
        /// <param name="dateRange">dateRange (required).</param>
        /// <param name="fieldList">fieldList (required).</param>
        /// <param name="history">history (required).</param>
        public HistoryPointDayObj(PointObj location = default(PointObj), DateRangeObj dateRange = default(DateRangeObj), FieldListDayObj fieldList = default(FieldListDayObj), HistoryDayArr history = default(HistoryDayArr))
        {
            // to ensure "location" is required (not null)
            if (location == null)
            {
                throw new InvalidDataException("location is a required property for HistoryPointDayObj and cannot be null");
            }
            else
            {
                this.Location = location;
            }
            // to ensure "dateRange" is required (not null)
            if (dateRange == null)
            {
                throw new InvalidDataException("dateRange is a required property for HistoryPointDayObj and cannot be null");
            }
            else
            {
                this.DateRange = dateRange;
            }
            // to ensure "fieldList" is required (not null)
            if (fieldList == null)
            {
                throw new InvalidDataException("fieldList is a required property for HistoryPointDayObj and cannot be null");
            }
            else
            {
                this.FieldList = fieldList;
            }
            // to ensure "history" is required (not null)
            if (history == null)
            {
                throw new InvalidDataException("history is a required property for HistoryPointDayObj and cannot be null");
            }
            else
            {
                this.History = history;
            }
        }
        
        /// <summary>
        /// Gets or Sets Location
        /// </summary>
        [DataMember(Name="location", EmitDefaultValue=false)]
        public PointObj Location { get; set; }

        /// <summary>
        /// Gets or Sets DateRange
        /// </summary>
        [DataMember(Name="dateRange", EmitDefaultValue=false)]
        public DateRangeObj DateRange { get; set; }

        /// <summary>
        /// Gets or Sets FieldList
        /// </summary>
        [DataMember(Name="fieldList", EmitDefaultValue=false)]
        public FieldListDayObj FieldList { get; set; }

        /// <summary>
        /// Gets or Sets History
        /// </summary>
        [DataMember(Name="history", EmitDefaultValue=false)]
        public HistoryDayArr History { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class HistoryPointDayObj {\n");
            sb.Append("  Location: ").Append(Location).Append("\n");
            sb.Append("  DateRange: ").Append(DateRange).Append("\n");
            sb.Append("  FieldList: ").Append(FieldList).Append("\n");
            sb.Append("  History: ").Append(History).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as HistoryPointDayObj);
        }

        /// <summary>
        /// Returns true if HistoryPointDayObj instances are equal
        /// </summary>
        /// <param name="input">Instance of HistoryPointDayObj to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(HistoryPointDayObj input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Location == input.Location ||
                    (this.Location != null &&
                    this.Location.Equals(input.Location))
                ) && 
                (
                    this.DateRange == input.DateRange ||
                    (this.DateRange != null &&
                    this.DateRange.Equals(input.DateRange))
                ) && 
                (
                    this.FieldList == input.FieldList ||
                    (this.FieldList != null &&
                    this.FieldList.Equals(input.FieldList))
                ) && 
                (
                    this.History == input.History ||
                    (this.History != null &&
                    this.History.Equals(input.History))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Location != null)
                    hashCode = hashCode * 59 + this.Location.GetHashCode();
                if (this.DateRange != null)
                    hashCode = hashCode * 59 + this.DateRange.GetHashCode();
                if (this.FieldList != null)
                    hashCode = hashCode * 59 + this.FieldList.GetHashCode();
                if (this.History != null)
                    hashCode = hashCode * 59 + this.History.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }
}
