/* 
 * Weather Source History API
 *
 * Daily and hourly historical weather data for latitude/longitude points, Zip/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source History API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */
using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// Hourly Postcode historical weather object
    /// </summary>
    [DataContract]
        public partial class HistoryPostcodeHourObj :  IEquatable<HistoryPostcodeHourObj>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="HistoryPostcodeHourObj" /> class.
        /// </summary>
        /// <param name="location">location (required).</param>
        /// <param name="timestampRange">timestampRange (required).</param>
        /// <param name="fieldList">fieldList (required).</param>
        /// <param name="history">history (required).</param>
        public HistoryPostcodeHourObj(PostcodeObj location = default(PostcodeObj), TimestampRangeObj timestampRange = default(TimestampRangeObj), FieldListHourObj fieldList = default(FieldListHourObj), HistoryHourArr history = default(HistoryHourArr))
        {
            // to ensure "location" is required (not null)
            if (location == null)
            {
                throw new InvalidDataException("location is a required property for HistoryPostcodeHourObj and cannot be null");
            }
            else
            {
                this.Location = location;
            }
            // to ensure "timestampRange" is required (not null)
            if (timestampRange == null)
            {
                throw new InvalidDataException("timestampRange is a required property for HistoryPostcodeHourObj and cannot be null");
            }
            else
            {
                this.TimestampRange = timestampRange;
            }
            // to ensure "fieldList" is required (not null)
            if (fieldList == null)
            {
                throw new InvalidDataException("fieldList is a required property for HistoryPostcodeHourObj and cannot be null");
            }
            else
            {
                this.FieldList = fieldList;
            }
            // to ensure "history" is required (not null)
            if (history == null)
            {
                throw new InvalidDataException("history is a required property for HistoryPostcodeHourObj and cannot be null");
            }
            else
            {
                this.History = history;
            }
        }
        
        /// <summary>
        /// Gets or Sets Location
        /// </summary>
        [DataMember(Name="location", EmitDefaultValue=false)]
        public PostcodeObj Location { get; set; }

        /// <summary>
        /// Gets or Sets TimestampRange
        /// </summary>
        [DataMember(Name="timestampRange", EmitDefaultValue=false)]
        public TimestampRangeObj TimestampRange { get; set; }

        /// <summary>
        /// Gets or Sets FieldList
        /// </summary>
        [DataMember(Name="fieldList", EmitDefaultValue=false)]
        public FieldListHourObj FieldList { get; set; }

        /// <summary>
        /// Gets or Sets History
        /// </summary>
        [DataMember(Name="history", EmitDefaultValue=false)]
        public HistoryHourArr History { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class HistoryPostcodeHourObj {\n");
            sb.Append("  Location: ").Append(Location).Append("\n");
            sb.Append("  TimestampRange: ").Append(TimestampRange).Append("\n");
            sb.Append("  FieldList: ").Append(FieldList).Append("\n");
            sb.Append("  History: ").Append(History).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as HistoryPostcodeHourObj);
        }

        /// <summary>
        /// Returns true if HistoryPostcodeHourObj instances are equal
        /// </summary>
        /// <param name="input">Instance of HistoryPostcodeHourObj to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(HistoryPostcodeHourObj input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Location == input.Location ||
                    (this.Location != null &&
                    this.Location.Equals(input.Location))
                ) && 
                (
                    this.TimestampRange == input.TimestampRange ||
                    (this.TimestampRange != null &&
                    this.TimestampRange.Equals(input.TimestampRange))
                ) && 
                (
                    this.FieldList == input.FieldList ||
                    (this.FieldList != null &&
                    this.FieldList.Equals(input.FieldList))
                ) && 
                (
                    this.History == input.History ||
                    (this.History != null &&
                    this.History.Equals(input.History))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Location != null)
                    hashCode = hashCode * 59 + this.Location.GetHashCode();
                if (this.TimestampRange != null)
                    hashCode = hashCode * 59 + this.TimestampRange.GetHashCode();
                if (this.FieldList != null)
                    hashCode = hashCode * 59 + this.FieldList.GetHashCode();
                if (this.History != null)
                    hashCode = hashCode * 59 + this.History.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }
}
