/* 
 * Weather Source History API
 *
 * Daily and hourly historical weather data for latitude/longitude points, Zip/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source History API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */
using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// Metadata object for a postcode
    /// </summary>
    [DataContract]
        public partial class PointObj :  IEquatable<PointObj>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PointObj" /> class.
        /// </summary>
        /// <param name="latitude">latitude (required).</param>
        /// <param name="longitude">longitude (required).</param>
        /// <param name="timezone">timezone (required).</param>
        /// <param name="countryCode">countryCode.</param>
        /// <param name="countryName">countryName.</param>
        /// <param name="subdivCode">subdivCode.</param>
        /// <param name="subdivName">subdivName.</param>
        /// <param name="boundingPoints">boundingPoints (required).</param>
        /// <param name="grid">grid (required).</param>
        public PointObj(Latitude latitude = default(Latitude), Longitude longitude = default(Longitude), Timezone timezone = default(Timezone), CountryCode countryCode = default(CountryCode), CountryName countryName = default(CountryName), SubdivCode subdivCode = default(SubdivCode), SubdivName subdivName = default(SubdivName), BoundingPoints boundingPoints = default(BoundingPoints), Grid grid = default(Grid))
        {
            // to ensure "latitude" is required (not null)
            if (latitude == null)
            {
                throw new InvalidDataException("latitude is a required property for PointObj and cannot be null");
            }
            else
            {
                this.Latitude = latitude;
            }
            // to ensure "longitude" is required (not null)
            if (longitude == null)
            {
                throw new InvalidDataException("longitude is a required property for PointObj and cannot be null");
            }
            else
            {
                this.Longitude = longitude;
            }
            // to ensure "timezone" is required (not null)
            if (timezone == null)
            {
                throw new InvalidDataException("timezone is a required property for PointObj and cannot be null");
            }
            else
            {
                this.Timezone = timezone;
            }
            // to ensure "boundingPoints" is required (not null)
            if (boundingPoints == null)
            {
                throw new InvalidDataException("boundingPoints is a required property for PointObj and cannot be null");
            }
            else
            {
                this.BoundingPoints = boundingPoints;
            }
            // to ensure "grid" is required (not null)
            if (grid == null)
            {
                throw new InvalidDataException("grid is a required property for PointObj and cannot be null");
            }
            else
            {
                this.Grid = grid;
            }
            this.CountryCode = countryCode;
            this.CountryName = countryName;
            this.SubdivCode = subdivCode;
            this.SubdivName = subdivName;
        }
        
        /// <summary>
        /// Gets or Sets Latitude
        /// </summary>
        [DataMember(Name="latitude", EmitDefaultValue=false)]
        public Latitude Latitude { get; set; }

        /// <summary>
        /// Gets or Sets Longitude
        /// </summary>
        [DataMember(Name="longitude", EmitDefaultValue=false)]
        public Longitude Longitude { get; set; }

        /// <summary>
        /// Gets or Sets Timezone
        /// </summary>
        [DataMember(Name="timezone", EmitDefaultValue=false)]
        public Timezone Timezone { get; set; }

        /// <summary>
        /// Gets or Sets CountryCode
        /// </summary>
        [DataMember(Name="countryCode", EmitDefaultValue=false)]
        public CountryCode CountryCode { get; set; }

        /// <summary>
        /// Gets or Sets CountryName
        /// </summary>
        [DataMember(Name="countryName", EmitDefaultValue=false)]
        public CountryName CountryName { get; set; }

        /// <summary>
        /// Gets or Sets SubdivCode
        /// </summary>
        [DataMember(Name="subdivCode", EmitDefaultValue=false)]
        public SubdivCode SubdivCode { get; set; }

        /// <summary>
        /// Gets or Sets SubdivName
        /// </summary>
        [DataMember(Name="subdivName", EmitDefaultValue=false)]
        public SubdivName SubdivName { get; set; }

        /// <summary>
        /// Gets or Sets BoundingPoints
        /// </summary>
        [DataMember(Name="boundingPoints", EmitDefaultValue=false)]
        public BoundingPoints BoundingPoints { get; set; }

        /// <summary>
        /// Gets or Sets Grid
        /// </summary>
        [DataMember(Name="grid", EmitDefaultValue=false)]
        public Grid Grid { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class PointObj {\n");
            sb.Append("  Latitude: ").Append(Latitude).Append("\n");
            sb.Append("  Longitude: ").Append(Longitude).Append("\n");
            sb.Append("  Timezone: ").Append(Timezone).Append("\n");
            sb.Append("  CountryCode: ").Append(CountryCode).Append("\n");
            sb.Append("  CountryName: ").Append(CountryName).Append("\n");
            sb.Append("  SubdivCode: ").Append(SubdivCode).Append("\n");
            sb.Append("  SubdivName: ").Append(SubdivName).Append("\n");
            sb.Append("  BoundingPoints: ").Append(BoundingPoints).Append("\n");
            sb.Append("  Grid: ").Append(Grid).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as PointObj);
        }

        /// <summary>
        /// Returns true if PointObj instances are equal
        /// </summary>
        /// <param name="input">Instance of PointObj to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(PointObj input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.Latitude == input.Latitude ||
                    (this.Latitude != null &&
                    this.Latitude.Equals(input.Latitude))
                ) && 
                (
                    this.Longitude == input.Longitude ||
                    (this.Longitude != null &&
                    this.Longitude.Equals(input.Longitude))
                ) && 
                (
                    this.Timezone == input.Timezone ||
                    (this.Timezone != null &&
                    this.Timezone.Equals(input.Timezone))
                ) && 
                (
                    this.CountryCode == input.CountryCode ||
                    (this.CountryCode != null &&
                    this.CountryCode.Equals(input.CountryCode))
                ) && 
                (
                    this.CountryName == input.CountryName ||
                    (this.CountryName != null &&
                    this.CountryName.Equals(input.CountryName))
                ) && 
                (
                    this.SubdivCode == input.SubdivCode ||
                    (this.SubdivCode != null &&
                    this.SubdivCode.Equals(input.SubdivCode))
                ) && 
                (
                    this.SubdivName == input.SubdivName ||
                    (this.SubdivName != null &&
                    this.SubdivName.Equals(input.SubdivName))
                ) && 
                (
                    this.BoundingPoints == input.BoundingPoints ||
                    (this.BoundingPoints != null &&
                    this.BoundingPoints.Equals(input.BoundingPoints))
                ) && 
                (
                    this.Grid == input.Grid ||
                    (this.Grid != null &&
                    this.Grid.Equals(input.Grid))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.Latitude != null)
                    hashCode = hashCode * 59 + this.Latitude.GetHashCode();
                if (this.Longitude != null)
                    hashCode = hashCode * 59 + this.Longitude.GetHashCode();
                if (this.Timezone != null)
                    hashCode = hashCode * 59 + this.Timezone.GetHashCode();
                if (this.CountryCode != null)
                    hashCode = hashCode * 59 + this.CountryCode.GetHashCode();
                if (this.CountryName != null)
                    hashCode = hashCode * 59 + this.CountryName.GetHashCode();
                if (this.SubdivCode != null)
                    hashCode = hashCode * 59 + this.SubdivCode.GetHashCode();
                if (this.SubdivName != null)
                    hashCode = hashCode * 59 + this.SubdivName.GetHashCode();
                if (this.BoundingPoints != null)
                    hashCode = hashCode * 59 + this.BoundingPoints.GetHashCode();
                if (this.Grid != null)
                    hashCode = hashCode * 59 + this.Grid.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }
}
